/***********************************************************************
	Module MCB.C
	Routines to manage DOS Memory Control Blocks.

	Copyright (C) 1994, Brian Sawert.
	All rights reserved.

	History:
		02/02/94 - BAS - original code

	Notes:
		Real mode only.  Do not use under Windows.
***********************************************************************/

#include <dos.h>
#include <string.h>

#include "mcb.h"

	
/* -------------------- defines and macros -------------------- */

#define FUNC_LIST		0x5200		/* DOS List of Lists function */
#define FUNC_UMB_STAT	0x5802		/* UMB link status function */
#define FUNC_UMB_LINK	0x5803		/* link UMB function */


/* -------------------- start of code -------------------- */

/*----------------------------------------------------------------------
	Routine to get base segment of allocation chain.

	Usage:	unsigned get_base_seg(void);

	Called with nothing.
	Returns segment of first MCB, 0 on error.
----------------------------------------------------------------------*/

unsigned get_base_seg(void)
{				
	union REGS params;
	struct SREGS segregs;
	unsigned far *segptr;
	unsigned retval = 0;

	params.x.ax = FUNC_LIST;

	intdosx(&params, &params, &segregs);	/* call DOS function */

	if (params.x.cflag == 0)
		{								/* call succeeded */
		segptr = (unsigned far *) MK_FP(segregs.es, params.x.bx - 2);
										/* point to word before List */
		retval = *segptr;
		}

	return(retval);
}



/*----------------------------------------------------------------------
	Routine to get next segment in allocation chain.

	Usage:	unsigned get_next_seg(unsigned seg, int f_umb);

	Called with current segment, flag to include upper memory.
	Returns segment of next MCB in chain, 0 if last MCB.
----------------------------------------------------------------------*/

unsigned get_next_seg(unsigned seg, int f_umb)
{				
	MCB far *lpmcb;
	unsigned next_seg = 0;

	lpmcb = (MCB far *) MK_FP(seg, 0);	/* point to MCB */

	if (lpmcb->mtype != 'Z' || (seg < 0xa000 && f_umb))
		{								/* intermediate link in chain */
		next_seg = seg + lpmcb->size + 1;
		}

	return(next_seg);
}



/*----------------------------------------------------------------------
	Routine to get MCB owner name

	Usage:	int mem_owner_name(unsigned seg, char *pname);

	Called with MCB segment, pointer to name buffer.
	Returns name length on success, 0 on failure.
----------------------------------------------------------------------*/

int mem_owner_name(unsigned seg, char *pname)
{
	MCB far *lpmcb;						/* pointer to MCB */
	char nbuff[NAME_LEN + 1];

	memset(nbuff, 0, sizeof(nbuff));	/* clear buffer */

	lpmcb = (MCB far *) MK_FP(seg, 0);	/* point to MCB */

	if (lpmcb->owner == seg + 1)
		{								/* first program block - valid name */
		_fstrncpy((char far *) nbuff, lpmcb->name, NAME_LEN);
		}

	strcpy(pname, nbuff);				/* copy from local buffer */

	return(strlen(pname));
}



/*----------------------------------------------------------------------
	Routine to find TSR in memory.

	Usage:	unsigned get_tsr_seg(char *pname, int f_umb);

	Called with pointer to TSR name, flag for upper memory search.
	Returns TSR segment on success, 0 on failure.
----------------------------------------------------------------------*/

unsigned get_tsr_seg(char *pname, int f_umb)
{				
	char buff[NAME_LEN + 1];
	unsigned seg;
	unsigned retval = 0;

	seg = get_base_seg();				/* get base segment */

	while (seg != 0)
		{								/* got legal value - continue */
		if (mem_owner_name(seg, buff) != 0)
			{							/* got valid owner name */
			if (stricmp(buff, pname) == 0)
				{						/* case insensitive match */
				retval = seg + 1;		/* TSR found */
				break;
				}
			}

		seg = get_next_seg(seg, f_umb);	/* get next header */
		}

	return(retval);
}
	


/*----------------------------------------------------------------------
	Routine to get status of DOS upper memory link.

	Usage:	int get_umb_link(void);

	Called with nothing.
	Returns 1 if upper memory linked, 0 otherwise.
----------------------------------------------------------------------*/

int get_umb_link(void)
{
	union REGS params;
	int retval;

	params.x.ax = FUNC_UMB_STAT;		/* get DOS upper memory link */
	intdos(&params, &params);

	retval = (int) params.h.al;			/* return link status */

	return(retval);
}



/*----------------------------------------------------------------------
	Routine to set DOS upper memory link status.

	Usage:	int set_umb_link(int f_link);

	Called with flag to link or unlink;
	Returns 0 on success, DOS error code on failure.
----------------------------------------------------------------------*/

int set_umb_link(int f_link)
{
	union REGS params;
	int retval;

	params.x.ax = FUNC_UMB_LINK;		/* set DOS upper memory link */
	params.x.bx = (f_link) ? 1 : 0;		/* set link flag */
	intdos(&params, &params);

	retval = (params.x.cflag == 0) ? 0 : params.x.ax;

	return(retval);
}

